# numbers.rb : Different ParameterTypes to deal with numbers
# Copyright (C) 2006 Vincent Fourmond
# 
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
# 
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301 USA

require 'time'

module MetaBuilder

  # The module ParameterTypes should be used for all subclasses of
  # ParameterType, to keep the place clean and tidy.
  module ParameterTypes

    # A combination date/time
    class DateTimeParameter < ParameterType


      # Converts a Time to a Variant.
      def type_to_variant(val)
        if val.is_a? Qt::DateTime
          return Qt::Variant.new(val)
        else
          epoch = Qt::Date.new(1970, 1, 1)
          date = Qt::DateTime.new(epoch)
          return Qt::Variant.new(date.add_secs(val.to_i))
        end
      end

      # Converts back to Time
      def variant_to_type(variant)
        epoch = Qt::DateTime.new(Qt::Date.new(1970, 1, 1))
        nb = epoch.secs_to(variant.value)
        return Time.at(nb)
      end

    end
  end

end
